{ Delphi Object for R&R SQL V6.5 Edition }

{ Written : October 1995 }
{ Property Editors added Jan 1996 }

{ TRRX65 & TRRSQL65 are written and distributed under the GNU 
  General Public License which means that its source code is 
  freely-distributed and available to the general public. }

{ Author Chris Brooksbank (cbrooksbank@dial.pipex.com) }
{ Web Page : http://ds.dial.pipex.com/cbrooksbank }

{ Copyright Chris Brooksbank 1995,1996,1997. }
{ 34 Vermeer Ride
  Springfield
  Chelmsford
  Essex
  CM1 6GA
  United Kingdom } 

unit RRSQL65;

interface

uses
  SysUtils,WinTypes,WinProcs,Messages,Classes,Graphics,Controls,Forms,
  Dialogs,rrsqlint, DsgnIntf, FileCtrl,rrsort,rruser,StdCtrls,rrxdir;


type
  ErrException=Class(Exception);
  ErrHandleAllocation=Class(ErrException);
  ErrCancel=Class(EAbort);
  ErrDiagnostic=Class(ErrException);
  ErrIteration=Class(ErrException);
  ErrJobControl=Class(ErrException);
  ErrLibrary=Class(ErrException);
  ErrSyntax=Class(ErrException);
  ErrValue=Class(ErrException);
  ErrUnKnown=Class(ErrException);
  ErrCodeAbort=Class(EAbort);

  { Custom event procedure types }
  TReportStartEvent=procedure(const ReportHandle:Integer;var AllowToRun:Boolean) of object;
  TReportEndEvent=procedure(const ReportHandle:Integer) of object;

  { Report Library Property Editor }
  TRLPropertyEditor=class(TPropertyEditor)
    public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Report Name Property Editor }
  TRNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Printer Name/Port Property Editor Class}
 TPRPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Data Directory Property Editor Class}
 TDDPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { Image Directory Property Editor Class}
 TIDPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

 { MemoName Property Editor Class}
 TMNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

   { Sortfields Property Editor Class }
  TSFPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Groupfields Property Editor Class }
  TGFPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { UserParamsNames/Values Property Editor Class }
  TUSPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { DataSource Property Editor Class }
  TDSPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Databasename Property Editor Class }
  TDNPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;

  { Filter Property Editor Class }
  TFIPropertyEditor=class(tPropertyEditor)
     public
      procedure edit; override;
      function GetAttributes: TPropertyAttributes; override;
      function GetValue:String; override;
      procedure SetValue(Const value:String); override;
  end;


  TrrFilterUsage = (rrfuSaved,rrfuNone,rrfuComponent,rrfuInteractive);
  TrrDest = (rrdDisplay,rrdTextFile,rrdPrinter,rrdWorksheet,rrdXBase,
             rrdInteractive,rrdCSV,rrdMSWord,rrdRTF);
  TrrExportDest = (rredDisplay,rredFile,rredPrinter);
  TrrwsBorderStyle = (rrwsNone,rrwsFixedSingle,rrwsSizable,rrwsFixedDouble);

  TRRSQL65 = class(TComponent)
  private
    { Private declarations }

     { SQL Specfic Fields Follow }
     fAbout:Boolean;
     fAskDataSource:Boolean;
     fAskTable:Boolean;
     fDataSource: String;
     fJoinTablenames: TStrings;
     fJoinAliasNames: TStrings;
     fPassword: String;
     fReplaces: TStrings;
     fUserName: String;
     fWhere:String;

     { Generic Fields Follow }
     fActive:Boolean;
     fAskPrinter:Boolean;
     fAskReport:Boolean;
     fAuthor:String;
     fAppName:String;
     fBeginPage: Longint;
     fCopies: Longint;
     fDatabasename: String;
     fDataDir    : String;
     fDisplayErrors: Boolean;
     fDisplayStatus: Boolean;
     fEndPage: LongInt;
     fErrorCode: String;
     fErrorMessage: String;
     fExportDest: TrrExportDest;
     fFields: TStrings;
     fFilter: String;
     fFilterLong:TStrings;
     fFilterUsage: TrrFilterUsage;
     fGroupFields: TStrings;
     fImageDir   : String;
     fLastErrorPage: Longint;
     fLibName: String;
     fMasterTableName: String;
     fMemoName: String;
     fOutputDest: TRRDest;
     fOutputFile: String;
     fPreventEscape: Boolean;
     fPrinterName: String;
     fPrinterPort: String;
     fRepName:String;
     fReportPick: Boolean;
     fSortFields: TStrings;
     fStatusEveryPage: Boolean;
     fSuppressTitle: Boolean;
     fTestPattern: Boolean;
     fUserParamsNames: TStrings;
     fUserParamsValues: TStrings;
     fVersion: String;
     fWait: Boolean;
     fWinBorderStyle: TrrwsBorderStyle;
     fWinControlBox: Boolean;
     fWinHeight: Integer;
     fWinLeft:Integer;
     fWinMaxButton: Boolean;
     fWinMinButton: Boolean;
     fWinParentHandle: Integer;
     fWinTitle: String;
     fWinTop: Integer;
     fWinWidth:Integer;

     { Event fields }
     fOnReportStart:TReportStartEvent;
     fOnReportEnd:TReportEndEvent;

     { SQL Specific private functions follow }

     procedure LoadJoins(hReport:Integer);
     procedure setfJoinTableNames(Value:Tstrings);
     procedure setfJoinAliasNames(Value:Tstrings);
     procedure setfReplaces(Value:Tstrings);
     procedure savereplaces(hReport:Integer);
     procedure loadreplaces(hReport:Integer);
     procedure loaddatasource(hReport:Integer);
     procedure LoadFromReportH(hMyReport:Integer);

     { Generic functions follow }

     procedure SetAbout(Value:Boolean);
     procedure setfActive(Value:Boolean);
     procedure setfAuthor(Value:String);
     procedure LoadFields(hReport:Integer);
     procedure LoadGroupFields(hReport:Integer);
     procedure LoadSortFields(hReport:Integer);
     procedure LoadUserParams(hReport:Integer);
     procedure LoadTitle(hReport:Integer);
     procedure LoadPages(hReport:Integer);
     procedure LoadTable(hReport:Integer);
     procedure LoadFilter(hReport:Integer);
     procedure LoadDests(hReport:Integer);
     procedure LoadPrinter(hReport:Integer);

     procedure setfFields(Value:Tstrings);
     procedure setfFilter(Value:String);
     procedure setfFilterLong(Value:Tstrings);
     procedure setfGroupFields(Value:Tstrings);
     procedure setfSortFields(Value:Tstrings);
     procedure setfUserParamsNames(Value:Tstrings);
     procedure setfUserParamsValues(Value:Tstrings);
     procedure SetLibName(NewLibName:String);
     procedure SetRepName(NewRepName: String);
     procedure SetVersion(Value:String);
     procedure SaveToReportH(hReport:Integer);

     procedure SortForm(SortPage:Boolean);
     procedure SortFormSave(GroupForm:TSortDialog);
     procedure SortFormLoad(GroupForm:TSortDialog);

  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner:TComponent); override;
    destructor destroy;override;
    function execute:Boolean;virtual;
    function run:Boolean;virtual;
    procedure LoadFromReport;virtual;
    function SaveToFile(MyFile:String):Boolean;virtual;
    function LoadFromFile(MyFile:String):Boolean;virtual;
    procedure Clear;virtual;
    procedure Preview;virtual;

    procedure SelectReportName;virtual;
    procedure SelectReportLibrary;virtual;
    procedure SelectPrinter;virtual;
    procedure SelectDataDirectory;virtual;
    procedure SelectImageDirectory;virtual;
    procedure SelectMemoName;virtual;
    procedure SelectSortFields;virtual;
    procedure SelectGroupFields;virtual;
    procedure SelectUserParams;virtual;
    procedure SelectDirs;virtual;
    procedure SelectDataSource;virtual;
    procedure SelectDatabase;virtual;
    procedure SelectFilter;virtual;
    procedure SelectJoins;virtual;
    function  SelectTable:String;virtual;

  published
    { Published declarations }

    { SQL Specific Properties Follow }

    property About:Boolean read fAbout write SetAbout;
    property AskDataSource:Boolean read fAskDataSource write fAskDataSource;
    property AskTable:Boolean read fAskTable write fAskTable;
    property DataSource: String read fDataSource write fDataSource;
    property JoinTableNames: TStrings read fJoinTableNames
                                      write setfJoinTableNames;
    property JoinAliasNames: TStrings read fJoinAliasNames
                                      write setfJoinAliasNames;
    property Password: String read fPassword write fPassword;
    property Replaces:Tstrings read fReplaces write SetfReplaces;
    property UserName: String read fUserName write fUserName;
    property Where: String read fWhere write fWhere;

    { Generic Properties follow }

    property Active: Boolean read fActive write SetfActive;
    property AppName: String read fAppName write fAppName;
    property AskPrinter:Boolean read fAskPrinter write fAskPrinter;
    property AskReport:Boolean read fAskReport write fAskReport;
    property Author: String read fAuthor write setfAuthor;
    property BeginPage: Longint read fBeginPage write fBeginPage;
    property Copies: Longint read fCopies write fCopies;
    property Databasename:String read fDatabasename write fDatabasename;
    property DataDirectory:String read fDataDir write fDataDir;
    property DisplayErrors: Boolean read fDisplayErrors write fDisplayErrors;
    property DisplayStatus: Boolean read fDisplayStatus write fDisplayStatus;
    property EndPage: Longint read fEndPage write fEndPage;
    property ErrorCode: String read fErrorcode write fErrorcode;
    property ErrorMessage: String read fErrorMessage write fErrorMessage;
    property ExportDest: TrrExportDest read fExportDest write fExportDest;
    property Fields: TStrings read fFields write setfFields;
    property Filter: String read fFilter write setfFilter;
    property FilterLong: TStrings read fFilterLong write setfFilterLong;
    property FilterUsage: TrrFilterUsage read fFilterUsage write fFilterUsage;
    property GroupFields: TStrings read fGroupFields write setfGroupFields;
    property ImageDir: String read fImageDir write fImageDir;
    property LastErrorPage:Longint read fLastErrorPage write fLastErrorPage;
    property MasterTableName: String read fMasterTableName write fMasterTableName;
    property MemoName: String read fMemoName write fMemoName;
    property OutputDest: TrrDest read fOutputDest write fOutputDest;
    property OutputFile: String read fOutputFile write fOutputFile;
    property PreventEscape: Boolean read fPreventEscape write fPreventEscape;
    property PrinterName: String read fPrinterName write fPrinterName;
    property PrinterPort: String read fPrinterPort write fPrinterPort;
    property ReportLibrary:String read fLibName write setLibName;
    property ReportName: String read fRepName write setRepName;
    property SortFields: TStrings read fSortFields write setfSortFields;
    property StatusEveryPage: Boolean read fStatusEveryPage
                                      write fStatusEveryPage;
    property SuppressTitle: Boolean read fSuppressTitle write fSuppressTitle;
    property TestPattern: Boolean read fTestPattern write fTestPattern;
    property UserParamsNames: TStrings read fUserParamsNames
                                       write setfUserParamsNames;
    property UserParamsValues: TStrings read fUserParamsValues
                                       write setfUserParamsValues;
    property Version:String read fVersion write setversion;
    property Wait: Boolean read fWait write fWait;
    property WinBorderStyle:TrrwsBorderStyle read fWinBorderStyle
             write fWinBorderStyle;
    property WinControlBox: Boolean read fWinControlBox write fWinControlBox;
    property WinHeight: Integer read fWinHeight write fWinHeight;
    property WinLeft: Integer read fWinLeft write fWinLeft;
    property WinMaxButton: Boolean read fWinMaxButton write fWinMaxButton;
    property WinMinButton: Boolean read fWinMinButton write fWinMinButton;
    property WinParentHandle: Integer read fWinParentHandle write fWinParentHandle;
    property WinTitle: String read fWinTitle write fWinTitle;
    property WinTop:Integer read fWinTop write fWinTop;
    property WinWidth:Integer read fWinWidth write fWinWidth;

   { Events follow }
   property OnReportStart:TReportStartEvent read fOnReportStart write fOnReportStart;
   property OnReportEnd:TReportEndEvent read fOnReportEnd write fOnReportEnd;
end;

procedure Register;

implementation


uses
  rrsqlabt,RRSQLFi,RRSRelat,rrsed;

{ ************************************************************************** }
constructor TRRSQL65.create(AOwner:Tcomponent);
{ ************************************************************************** }
begin
  inherited create(AOwner);

  initruntimeinstance;

  fversion:='2.3';

  { SQl Specific TSTRING fields }
  fJoinTableNames:=TStringList.Create;
  fJoinAliasNames:=TStringList.Create;
  fReplaces:=TStringList.Create;

  { Generic TSTRING fields }
  fFields:=TStringList.Create;
  fFilterLong:=TStringList.Create;
  fGroupFields:=TStringList.Create;
  fSortFields:=TstringList.Create;
  fUserParamsNames:=TStringList.Create;
  fUserParamsValues:=TStringList.Create;

  { Some defaults I thought looked sensible }
  fWait:=true;
  fAppName:=Application.exename;
  fAskPrinter:=false;
  fLastErrorPage:=0;
  WinTitle:='Delphi/R&R Object - (C) Chris Brooksbank 1995,1996,1997';
  WinControlBox:=true;
  WinBorderStyle:=rrwsSizable;
  WinMinButton:=true;
  WinMaxButton:=true;
  WinLeft:=30;
  WinTop:=30;
  WinWidth:=Screen.Width div 2;
  WinHeight:=Screen.Height div 2 ;
  WinParentHandle:=0;

end;

{ ************************************************************************** }
destructor TRRSQL65.Destroy;
{ ************************************************************************** }
begin

  { SQL Specific TSTRING fields }
  fJoinTableNames.Free;
  fJoinAliasNames.Free;
  fReplaces.free;

  { Generic TSTRING Fields }
  fFields.free;
  fFilterLong.Free;
  fGroupFields.free;
  fSortFields.free;
  fUserParamsNames.free;
  fUserParamsValues.free;

  endruntimeinstance;

  inherited Destroy;
end;


{ ************************************************************************** }
procedure Register;
{ ************************************************************************** }
begin
  RegisterComponents('Data Access', [TRRSQL65]);
  RegisterComponentEditor(TRRSQL65,TRRSQL65Editor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'ReportLibrary',
                         TRLPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'ReportName',
                         TRNPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'PrinterName',
                         TPRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'PrinterPort',
                         TPRPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'DataDirectory',
                         TDDPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'ImageDir',
                         TIDPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'MemoName',
                         TMNPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRSQL65,'SortFields',
                         TSFPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRSQL65,'GroupFields',
                         TGFPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRSQL65,'UserParamsNames',
                         TUSPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRSQL65,'UserParamsValues',
                         TUSPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'DataSource',
                         TDSPropertyEditor);
  RegisterPropertyEditor(TypeInfo(String),TRRSQL65,'DatabaseName',
                         TDNPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings),TRRSQL65,'Filter',
                         TFIPropertyEditor);

end;

{ ************************************************************************** }
procedure TRRSQL65.LoadFromReport;
{ ************************************************************************** }
{ Using the fRepname and fLibname load object info from report defaults }
var
  hMyReport:Integer;
  MyApp_,MyLib_,MyRep_: Pchar;
begin
  MyApp_:=StrAlloc(256);
  MyLib_:=StrAlloc(256);
  MyRep_:=StrAlloc(256);
  try

    if ((flibname<>'') and (frepname<>'')) then
    begin
      StrPCopy(MyApp_,fAppName);
      StrPCopy(MyLib_,fLibName);
      StrPCopy(MyRep_,fRepName);

      hMyReport:=ChooseReport(MyApp_,MyLib_,MyRep_,StrBufSize(MyRep_));
      if hMyReport>0 then
      begin
        try
          LoadFromReportH(hMyReport);
        finally
          EndReport(hMyReport);
        end;
      end;

      if hMyReport<1 then
        raise ErrHandleAllocation.Create('Cant load info on R&&R report '+ReportName);
    end;
  finally
    StrDispose(MyApp_);
    StrDispose(MyLib_);
    StrDispose(MyRep_);
  end;
end;


{ ************************************************************************** }
procedure TRRSQL65.LoadFromReportH(hMyReport:Integer);
{ ************************************************************************** }

{ Using passed report handle load object info from report defaults }

begin

  { Clear SQL Specific TSTRING Fields }
  fJoinTableNames.Clear;
  fJoinAliasNames.Clear;
  fReplaces.Clear;

  { Clear Generic TSTRING Fields }
  fFields.Clear;
  fGroupFields.Clear;
  fSortFields.Clear;
  fUserParamsNames.Clear;
  fUserParamsValues.Clear;

  { Load SQL Specific Properties }
  LoadJoins(hMyReport);
  LoadReplaces(hMyReport);
  LoadDataSource(hMyReport);

  { Load Generic Properties }
  LoadFields(hMyReport);
  LoadGroupFields(hMyReport);
  LoadSortFields(hMyReport);
  LoadUserParams(hMyReport);
  LoadTitle(hMyReport);
  LoadPages(hMyReport);
  LoadTable(hMyReport);
  LoadFilter(hMyReport);
  LoadDests(hMyReport);
  LoadPrinter(hMyReport);

end;

{ ************************************************************************** }
procedure TRRSQL65.setlibname(NewLibName:String);
{ ************************************************************************** }
var
  Designing:Boolean;
begin
  flibname:=NewLibName;
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if (Designing and (fLibName<>'') and (fRepName<>'') ) then
  begin
    if MessageDlg('Load Properties From Report ?',mtconfirmation,
                  [mbNo,mbYes],0)=mrYes then loadfromreport;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadJoins(hReport:Integer);
{ ************************************************************************** }
var
  JoinTableName,JoinAliasName:Pchar;
begin
  JoinTableName:=StrAlloc(51);
  JoinAliasName:=StrAlloc(51);
  try
    if GetFirstJoinInfo(hReport,JoinTableName,StrBufSize(JoinTableName),
                     JoinAliasName,StrBufSize(JoinAliasName)) then
    begin
      fJoinTableNames.Add(StrPas(JoinTableName));
      fJoinAliasNames.Add(StrPas(JoinAliasName));
    end;

    while GetNextJoinInfo(hReport,JoinTableName,StrBufSize(JoinTableName),
                          JoinAliasName,StrBufSize(JoinAliasName)) do begin
      fJoinTableNames.Add(StrPas(JoinTableName));
      fJoinAliasNames.Add(StrPas(JoinAliasName));
    end;
  finally
    StrDispose(JoinTableName);
    StrDispose(JoinAliasName);
  end;
end;

{ ************************************************************************** }
function TRRSQL65.Run:Boolean;
{ ************************************************************************** }
begin
  Result:=Execute;
end;


{ ************************************************************************** }
function TRRSQL65.Execute:Boolean;
{ ************************************************************************** }
var
  { Handle of report }
  hReport: Integer;

  { Flags returned after report was run }
  ECode:Integer;
  cmdshow: Integer;
  PageCount:LongInt;
  EMsg:Pchar;
  ErrorMess: String;

  AppName_,LibName_,RepName_:PChar;
  {text_,caption_:PChar;}
  ExecResult,Allow2Run:Boolean;
begin

  EMsg:=StrAlloc(256);
  AppName_:=StrAlloc(256);
  LibName_:=StrAlloc(256);
  RepName_:=StrAlloc(256);
  try

    {Run the report }
    Result:=false;

    { Convert Pascal type strings to C++ strings as expected by DLL }
    StrPCopy(Appname_,fAppname);
    StrPCopy(Libname_,fLibName);
    StrPCopy(RepName_,fRepName);

    { If object specifys ask for report then blank out the report name }
    { to make sure user is prompted for report name }
    if fAskReport then  begin
      fRepname:='';
      StrPCopy(RepName_,fRepName);
    end;

    hReport:=ChooseReport(Appname_,LibName_,RepName_,StrBufSize(RepName_));

    if hReport>0 then begin
      try
        { Pass all the propertys to RSREPORT.DLL }
        SaveToReportH(hReport);

        { Run the report and then clean up }
        cmdshow:=SW_SHOWNORMAL;
        fErrorCode:='';
        fErrorMessage:='';
        ResetErrorInfo;

        { Run custom start event, report then custom end event }
        Allow2Run:=true;
        if Assigned(fOnReportStart) then fOnReportStart(hReport,Allow2Run);
        if (not Allow2Run) then Raise ErrCodeAbort.Create('');
        ExecResult:= ExecRunTime(hReport,fWait,cmdshow,@ECode,
                     @PageCount,EMsg,StrBufSize(EMsg));
        if Assigned(fOnReportEnd) then fOnReportEnd(hReport);

        if ExecResult then Result:=True else begin
          geterrorinfo(Emsg,StrBufSize(EMsg),@Ecode);
          fErrorMessage:=StrPas(EMsg);

          case Ecode of
            Ord('C'):fErrorCode:='C';
            Ord('D'):fErrorCode:='D';
            Ord('I'):fErrorCode:='I';
            Ord('J'):fErrorCode:='J';
            Ord('L'):fErrorCode:='L';
            Ord('S'):fErrorCode:='S';
            Ord('V'):fErrorCode:='V';
            else
              fErrorCode:=Chr(Ecode);
          end;

          case Ecode of
            Ord('C'):raise ErrCancel.Create(fErrorMessage);
            Ord('D'):raise ErrDiagnostic.Create(fErrorMessage);
            Ord('I'):raise ErrIteration.Create(fErrorMessage);
            Ord('J'):raise ErrJobControl.Create(fErrorMessage);
            Ord('L'):raise ErrLibrary.Create(fErrorMessage);
            Ord('S'):raise ErrSyntax.Create(fErrorMessage);
            Ord('V'):raise ErrValue.Create(fErrorMessage);
            else
              raise ErrUnknown.Create(fErrorMessage);
          end;

        end;
      finally
        EndReport(hReport);
      end;
    end;
    fLastErrorPage:=PageCount;
    if hReport<1 then
      raise ErrHandleAllocation.Create('Cant allocate handle for report '+fRepname);
  finally
    StrDispose(EMsg);
    StrDispose(AppName_);
    StrDispose(LibName_);
    StrDispose(RepName_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfJoinTableNames(Value:Tstrings);
{ ************************************************************************** }
begin
  fJoinTableNames.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfJoinAliasNames(Value:Tstrings);
{ ************************************************************************** }
begin
  fJoinAliasNames.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfReplaces(Value:Tstrings);
{ ************************************************************************** }
begin
  fReplaces.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SaveReplaces(hReport:Integer);
{ ************************************************************************** }
var
  i:Integer;
  ThisReplace:String;
  Replaces_:Pchar;
  ThisReplace_:PChar;
begin
  Replaces_:=StrAlloc(4096);
  ThisReplace_:=StrAlloc(2048);
  try

    for i:=0 to (fReplaces.count-1) do begin
      ThisReplace:=fReplaces[i];
      if ThisReplace<>'' then begin
        StrPCopy(ThisReplace_,ThisReplace);
        StrLCat(Replaces_,ThisReplace_,StrBufSize(Replaces_));
      end;
    end;

    SetReplace(hReport,Replaces_);

  finally
    StrDispose(Replaces_);
    StrDispose(ThisReplace_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadReplaces(hReport:Integer);
{ ************************************************************************** }
var
  Replace:Pchar;
begin
  Replace:=StrAlloc(256);
  try
    if GetFirstReplace(hReport,Replace,StrBufSize(Replace)) then
      fReplaces.Add(StrPas(Replace));
    while GetNextReplace(hReport,Replace,StrBufSize(Replace)) do
      fReplaces.Add(StrPas(Replace));
  finally
   StrDispose(Replace);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadDataSource(hReport:Integer);
{ ************************************************************************** }
var
  Source_:Pchar;
begin
  Source_:=StrAlloc(256);
  try
    if GetDataSource(hReport,source_,StrBufSize(source_)) then
      fDataSource:=StrPas(Source_);
  finally
    StrDispose(Source_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadUserParams(hReport:Integer);
{ ************************************************************************** }
var
  ParamName,ParamValue:Pchar;
begin
  ParamName:=StrAlloc(31);
  ParamValue:=StrAlloc(31);
  try
    if GetFirstUserParam(hReport,ParamName,StrBufSize(ParamName),
                      paramValue,StrBufSize(ParamValue)) then begin
      fUserParamsNames.Add(StrPas(ParamName));
      fUserParamsValues.Add(StrPas(ParamValue));
    end;
    while GetNextUserParam(hReport,ParamName,StrBufSize(ParamName),
                           ParamValue,StrBufSize(ParamValue)) do begin
      fUserParamsNames.Add(StrPas(ParamName));
      fUSerParamsValues.Add(StrPas(ParamValue));
    end;
  finally
    StrDispose(Paramname);
    StrDispose(Paramvalue);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadFields(hReport:Integer);
{ ************************************************************************** }
var
  FieldName:Pchar;
begin
  FieldName:=StrAlloc(31);
  try
    if GetFirstFieldName(hReport,FieldName,StrBufSize(FieldName)) then
      fFields.Add(StrPas(FieldName));
    while GetNextFieldName(hReport,FieldName,StrBufSize(FieldName)) do
      fFields.Add(StrPas(FieldName));
  finally
    StrDispose(FieldName);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadGroupFields(hReport:Integer);
{ ************************************************************************** }
var
  GroupField:Pchar;
begin
  GroupField:=StrAlloc(31);
  try
    if GetFirstGroupField(hReport,GroupField,StrBufSize(GroupField)) then
      fGroupFields.Add(StrPas(GroupField));
    while GetNextGroupField(hReport,GroupField,StrBufSize(GroupField)) do
      fGroupFields.Add(StrPas(GroupField));
  finally
    StrDispose(GroupField);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadSortFields(hReport:Integer);
{ ************************************************************************** }
var
  SortField:Pchar;
begin
  SortField:=StrAlloc(31);
  try
    if GetFirstSortField(hReport,SortField,StrBufSize(SortField)) then
      fSortFields.Add(StrPas(SortField));
    while GetNextSortField(hReport,SortField,StrBufSize(SortField)) do
      fSortFields.Add(StrPas(SortField));
  finally
    StrDispose(SortField);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetRepName(NewRepName: String);
{ ************************************************************************** }
var
  Designing:Boolean;
begin
  fRepName:=NewRepName;
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if (Designing and (fLibName<>'') and (fRepName<>'') ) then
  begin
    if MessageDlg('Load Properties From Report ?',mtconfirmation,
                  [mbYes,mbNo],0)=mrYes then loadfromreport;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfFields(Value:Tstrings);
{ ************************************************************************** }
begin
  fFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfFilter(Value:String);
{ ************************************************************************** }
{ Set method for filter property. Also synchs FilterLong }
begin
  fFilter:=Value;
  if fFilterLong.Count=0 then
    fFilterLong.Add(Value)
  else
    fFilterLong[0]:=Value;
end;


{ ************************************************************************** }
procedure TRRSQL65.SetfGroupFields(Value:Tstrings);
{ ************************************************************************** }
begin
  fGroupFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfFilterLong(Value:Tstrings);
{ ************************************************************************** }
{ Set method for FilterLong property. This controls query of report.
  synchs with ffilter }
begin
  fFilterLong.Assign(Value);
  if fFilterLong.Count=0 then
    fFilter:=''
  else
    fFilter:=fFilterLong[0];
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfSortFields(Value:Tstrings);
{ ************************************************************************** }
begin
  fSortFields.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfUserParamsNames(Value:Tstrings);
{ ************************************************************************** }
begin
  fUserParamsNames.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfUserParamsValues(Value:Tstrings);
{ ************************************************************************** }
begin
  fUserParamsValues.Assign(Value);
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfActive(Value:Boolean);
{ ************************************************************************** }
begin
  if (csDesigning in ComponentState) and (Value=True) then execute;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetAbout(Value:Boolean);
{ ************************************************************************** }
{ Dummy set method for About property. Pops up about box with toolbar. }
var
  AboutBox:TRRSqlAbout;
  Designing:Boolean;
begin
  Designing:=(csDesigning in ComponentState);
  if ((csReading in ComponentState) or (csLoading in ComponentState)) then
    Designing:=false;

  if Designing then begin
    AboutBox:=TRRSQLAbout.Create(Application);
    AboutBox.Report:=self;
    AboutBox.ShowModal;
    AboutBox.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetfAuthor(Value:String);
{ ************************************************************************** }
begin
  if Value<>'cbrooksbank@dial.pipex.com' then
  messagedlg('Please send bugs/comments/enhancements to cbrooksbank@dial.pipex.com',mtInformation,
              [mbOk],0);
  fAuthor:='cbrooksbank@dial.pipex.com';
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadTitle(hReport:Integer);
{ ************************************************************************** }
var
  title_:Pchar;
begin
  title_:=StrAlloc(256);
  try
    if GetWinTitle(hReport,title_,StrBufSize(title_)) then
      fWinTitle:=StrPas(title_);
  finally
    StrDispose(title_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadPages(hReport:Integer);
{ ************************************************************************** }
begin
  GetBeginPage(hReport,@fBeginPage);
  GetEndPage(hReport,@fEndPage);
  GetCopies(hReport,@fCopies);
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadTable(hReport:Integer);
{ ************************************************************************** }
var
  table_:Pchar;
begin
  Table_:=StrAlloc(256);
  try
    if GetMasterTableName(hReport,table_,StrBufSize(table_)) then
      fMasterTableName:=StrPas(table_);
  finally
    StrDispose(Table_);
  end;
end;


{ ************************************************************************** }
procedure TRRSQL65.LoadFilter(hReport:Integer);
{ ************************************************************************** }
var
  filter_:Pchar;
begin
  filter_:=StrAlloc(256);
  try
    if getFilter(hReport,filter_,StrBufSize(filter_)) then
      fFilter:=StrPas(filter_);
  finally
    StrDispose(filter_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.LoadDests(hReport:Integer);
{ ************************************************************************** }
var
  Dest_:Pchar;
  DestStr:String;
begin
  Dest_:=StrAlloc(256);
  try

    { Load fExportDest}
    if getExportDest(hReport,Dest_) then
    begin
      DestStr:=StrPas(Dest_);
      if DestStr[1]='D' then
        fExportDest:=rredDisplay;
      if DestStr[1]='F' then
        fExportDest:=rredFile;
      if DestStr[1]='P' then
        fExportDest:=rredPrinter;
    end;

    { Load fOutPutDest }
    if getOutputDest(hReport,Dest_) then
    begin
      DestStr:=StrPas(Dest_);
      if DestStr[1]='D' then
        fOutPutDest:=rrdDisplay;
      if DestStr[1]='A' then
        fOutPutDest:=rrdTextFile;
      if DestStr[1]='T' then
        fOutPutDest:=rrdTextFile;
      if DestStr[1]='P' then
        fOutPutDest:=rrdPrinter;
      if DestStr[1]='W' then
        fOutPutDest:=rrdWorksheet;
      if DestStr[1]='X' then
        fOutPutDest:=rrdXBase;
      if DestStr[1]='?' then
        fOutPutDest:=rrdInteractive;
    end;

    { Load fOutputFile }
    if getOutputDest(hReport,Dest_) then
      fOutPutFile:=StrPas(Dest_);
  finally
    StrDispose(Dest_);
  end;

end;

{ ************************************************************************** }
procedure TRRSQL65.LoadPrinter(hReport:Integer);
{ ************************************************************************** }
var
  Printer_,Port_:Pchar;
begin
  Printer_:=StrAlloc(256);
  Port_:=StrAlloc(256);
  try
    if getPrinter(hReport,Printer_,StrBufSize(Printer_)) then
      fPrinterName:=StrPas(Printer_);
    if getPrinterPort(hReport,Port_,StrBufSize(Port_)) then
      fPrinterPort:=StrPas(Port_);
  finally
    StrDispose(Printer_);
    StrDispose(Port_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SaveToReportH(hReport:Integer);
{ ************************************************************************** }

{ Given a report handle save object properties to the report }
{ Pass all the propertys to RSREPORT.DLL }

var
  { Temp vars to hold Pchar versions of String properties }
  DataSource_,PrinterName_:PChar;
  PrinterPort_,PassWord_,UserName_:PChar;
  filter_,filterbit_:PChar;
  MasterTableName_,MemoName_,OutputFile_,WinTitle_: Pchar;
  Tablename_,Databasename_,Where_:Pchar;

  CharString:String;
  SiField,SiField2:Pchar;
  datadir_,imagedir_:Pchar;

  i,maxi:Integer;

begin
  DataSource_:=StrAlloc(256);
  PrinterName_:=StrAlloc(256);
  PrinterPort_:=StrAlloc(31);
  PassWord_:=StrAlloc(31);
  UserName_:=StrAlloc(31);
  filter_:=StrAlloc(256);
  filterbit_:=StrAlloc(256);
  MasterTableName_:=StrAlloc(256);
  MemoName_:=StrAlloc(256);
  OutputFile_:=StrAlloc(256);
  WinTitle_:=StrAlloc(256);
  TableName_:=StrAlloc(256);
  DatabaseName_:=StrAlloc(256);
  Where_:=StrAlloc(256);
  SiField:=StrAlloc(256);
  SiField2:=StrAlloc(256);
  datadir_:=StrAlloc(256);
  imagedir_:=StrAlloc(256);
  try

    if AskPrinter then fPrinterName:='?';

    { Convert Pascal type strings to C++ strings as expected by DLL }
    if fFilterLong.count=0 then
      StrPCopy(Filter_,fFilter)
    else begin
      StrPCopy(Filter_,fFilterLong[0]);
      for i:=1 to (fFilterLong.Count-1) do begin
        StrPCopy(FilterBit_,fFilterLong[i]);
        StrLCat(Filter_,filterbit_,1024);
      end;
    end;

    { Convert Pascal type strings to C++ strings as expected by DLL }
    StrPCopy(DataSource_,fDataSource);
    StrPCopy(MasterTableName_,fMasterTableName);
    StrPCopy(MemoName_,fMemoName);
    StrPCopy(OutputFile_,fOutputFile);
    StrPCopy(Password_,fPassword);
    StrPCopy(Printername_,fPrinterName);
    StrPCopy(PrinterPort_,fPrinterPort);
    StrPCopy(UserName_,fUserName);
    StrPCopy(WinTitle_,fWinTitle);
    StrPCopy(Databasename_,fDatabasename);
    StrPCopy(Where_,fwhere);
    StrPCopy(ImageDir_,fimagedir);
    StrPCopy(DataDir_,fdatadir);

    SetBeginPage(hReport,fBeginPage);
    SetCopies(hReport,fCopies);
    SetDisplayErrors(hReport,fDisplayErrors);
    SetDisplayStatus(hReport,FDisplayStatus);
    SetEndPage(hReport,fEndPage);

    CharString:=Copy('DFP',Ord(fExportDest)+1,1);
    SetExportDest(hReport,CharString[1]);

    SetFilter(hReport,Filter_);

    CharString:=Copy('SEO?',Ord(fFilterUsage)+1,1);
    SetFilterUsage(hReport,CharString[1]);

    if fMasterTableName<>''then
      SetMasterTableName(hReport,MasterTableName_);

    if fMemoName<>'' then
      SetMemoName(hReport,MemoName_);

    CharString:=Copy('DAPWX?CMR',Ord(fOutputDest)+1,1);
    SetOutputDest(hReport,CharString[1]);

    SetOutPutFile(hReport,OutputFile_);
    SetPassword(hReport,Password_);
    SetPreventEscape(hReport,fPreventEscape);
    SetStatusEveryPage(hReport,fStatusEveryPage);
    SetTestPattern(hReport,fTestPattern);
    SetUserName(hReport,Username_);

    CharString:=Copy('0123',Ord(fWinBorderStyle)+1,1);
    SetWinBorderStyle(hReport,Ord(CharString[1])-Ord('0'));

    SetWinControlBox(hReport,fWinControlBox);
    SetWinHeight(hReport,fWinHeight);
    SetWinLeft(hReport,fWinLeft);
    SetWinMaxButton(hReport,fWinMaxButton);
    SetWinMinButton(hReport,fWinMinButton);
    SetWinParentHandle(hReport,fWinParentHandle);
    SetWinTitle(hReport,WinTitle_);
    SetWinTop(hReport,fWinTop);
    SetWinWidth(hReport,fWinWidth);

    { Set sort fields }
    for i:=0 to (fSortFields.Count-1) do begin
      if ((fSortFields[i]<>'') and
         (Pos('RECNO',UpperCase(fSortFields[i]))=0)) then begin
        StrPCopy(SiField,fSortFields[i]);
        SetSortField(hReport,SiField,i+1);
      end;
    end;

    { Set Group Fields }
    for i:=0 to (fGroupFields.Count-1) do begin
      if fGroupFields[i]<>'' then begin
        StrPCopy(SiField,fGroupFields[i]);
        SetGroupField(hReport,SiField,i+1);
      end;
    end;

    { Set join information }
    maxi:=fJoinTableNames.count;
    if fJoinAliasNames.Count<maxi then maxi:=fJoinAliasNames.Count;
    dec(maxi);
    for i:=0 to maxi do begin
      if fJoinTableNames[i]<>'' then begin
        StrPCopy(SiField,fJoinTableNames[i]);
        StrPCopy(SiField2,fJoinAliasNames[i]);
        SetJoinInfo(hReport,SiField,Sifield2,i+1);
      end;
    end;

    { Set user paramaters }
    for i:=0 to (fUserParamsNames.Count-1) do begin
      if fUserParamsNames[i]<>'' then begin
        StrPCopy(SiField,fUserParamsNames[i]);
        StrPCopy(SiField2,fUserParamsValues[i]);
        SetUserParam(hreport,SiField,SiField2);
      end;
    end;

    if ((fPrinterName<>'') or AskPrinter) then
    begin
      SetPrinter(hReport,PrinterName_);
      SetPrinterPort(hReport,PrinterPort_);
    end;

    if AskDataSource then
      ChooseDataSource(hReport,Datasource_,StrBufSize(Datasource_));


    if AskTable then ChooseTable(hReport,Tablename_,StrBufSize(TableName_),
                                 DataSource_,StrBufSize(DataSource_),
                                 Databasename_,StrBufSize(Databasename_));

    if fDataSource<>'' then SetDataSource(hReport,DataSource_);

    if fwhere<>'' then SetWhere(hReport,where_);

    if fimagedir<>'' then SetImageDir(hreport,imagedir_);
    if fdatadir<>'' then SetDataDir(hreport,datadir_);

    SaveReplaces(hReport);
  finally
    StrDispose(DataSource_);
    StrDispose(PrinterName_);
    StrDispose(PrinterPort_);
    StrDispose(PassWord_);
    StrDispose(UserName_);
    StrDispose(filter_);
    StrDispose(filterbit_);
    StrDispose(MasterTableName_);
    StrDispose(MemoName_);
    StrDispose(Outputfile_);
    StrDispose(WinTitle_);
    StrDispose(TableName_);
    StrDispose(DatabaseName_);
    StrDispose(Where_);
    StrDispose(SiField);
    StrDispose(SiField2);
    StrDispose(datadir_);
    StrDispose(imagedir_);
 end;

end;


{ ************************************************************************** }
function TRRSQL65.SaveToFile(MyFile:String):Boolean;
{ ************************************************************************** }
{Save properties to runtime extended ASCII control file}
var
  hReport,fh:Integer;
  MyFile_,MyApp_,MyLib_,MyRep_:PChar;
begin
  MyFile_:=StrAlloc(256);
  MyApp_:=StrAlloc(256);
  MyLib_:=StrAlloc(256);
  MyRep_:=StrAlloc(256);
  try

    Result:=false;
    StrPCopy(MyApp_,fAppName);
    StrPCopy(MyLib_,fLibName);
    StrPCopy(MyRep_,fRepName);

    hReport:=ChooseReport(MyApp_,MyLib_,MyRep_,StrBufSize(MyRep_));
    try
      if hReport>0 then
      begin
        StrPCopy(MyFile_,MyFile);
        savetoreportH(hReport);

        fh:=filecreate(MyFile);
        if fh>0 then begin
          fileclose(fh);
          if writeRunTimeRecord(hReport,MyFile_) then result:=true;
        end;

      end;
    finally
      EndReport(hReport);
    end;
  finally
    StrDispose(MyFile_);
    StrDispose(MyApp_);
    StrDispose(MyLib_);
    StrDispose(MyRep_);
  end;
end;

{ ************************************************************************** }
function TRRSQL65.LoadFromFile(MyFile:String):Boolean;
{ ************************************************************************** }
{Load properties from runtime extended ASCII control file}
var
  hReport:Integer;
  MyFile_,MyApp_,MyLib_,MyRep_:PChar;
begin
  MyFile_:=StrAlloc(256);
  MyApp_:=StrAlloc(256);
  MyLib_:=StrAlloc(256);
  MyRep_:=StrAlloc(256);
  try
    Result:=false;
    StrPCopy(MyApp_,fAppName);
    StrPCopy(MyLib_,fLibName);
    StrPCopy(MyRep_,fRepName);

    StrPCopy(MyFile_,MyFile);
    hReport:=getRunTimeRecord(MyApp_,MyFile_);
    if hReport>0 then
    begin
      try
        { Load report information into this object }
        Result:=true;
        loadfromreporth(hReport);
      finally
        EndReport(hReport);
      end;
    end;
  finally
    StrDispose(MyFile_);
    StrDispose(MyApp_);
    StrDispose(MyLib_);
    StrDispose(MyRep_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectReportName;
{ ************************************************************************** }
{ allow user to choose a report }
var
  hReport:Integer;
  AppName_,ReportLibrary_,ReportName_:PChar;
begin
 if ReportLibrary='' then SelectReportLibrary;

  AppName_       := StrAlloc(length(AppName)+1);
  ReportLibrary_ := StrAlloc(length(ReportLibrary)+1);
  ReportName_    := StrAlloc(256);

  try
    StrPCopy(AppName_,AppName);
    StrPCopy(ReportLibrary_,ReportLibrary);
    StrPCopy(ReportName_,'');
    hReport:=ChooseReport(AppName_,ReportLibrary_,ReportName_,StrBufSize(ReportName_));

    if hReport>0 then begin
      ReportName:=StrPas(ReportName_);
      endReport(hReport);
    end;

  finally
    StrDispose(AppName_);
    StrDispose(ReportLibrary_);
    StrDispose(ReportName_);
  end;

end;

{ ************************************************************************** }
procedure TRRSQL65.SelectReportLibrary;
{ ************************************************************************** }
{ Allow user to select report library }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Report Librarys|*.RP6';
  if MyOpenDialog.execute then ReportLibrary:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectPrinter;
{ ************************************************************************** }
{ Allow user to select report printer }
var
  hReport:Integer;
  PrinterName_,PrinterPort_,AppName_:PChar;
begin
 PrinterName_ := StrAlloc(256);
 PrinterPort_ := StrAlloc(256);
 AppName_     := StrAlloc(256);

 try
   StrPCopy(PrinterName_,PrinterName);
   StrPCopy(PrinterPort_,PrinterName);
   StrPCopy(AppName_,AppName);

   hReport:=GetNewReportHandle(AppName_);

   if hReport>0 then begin
     try
         if ChoosePrinter(hReport,PrinterName_,StrBufSize(PrinterName_),
                              PrinterPort_,StrBufSize(PrinterPort_))
         then begin
           PrinterName:=StrPas(PrinterName_);
           PrinterPort:=StrPas(PrinterPort_);
         end;
     finally
       endreport(hReport);
     end;
   end;

 finally
   StrDispose(PrinterName_);
   StrDispose(PrinterPort_);
   StrDispose(AppName_);
 end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectDataDirectory;
{ ************************************************************************** }
{ Allow user to select datadirectory }
begin
  SelectDirectory(fDataDir,[],0);
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectImageDirectory;
{ ************************************************************************** }
{ Allow user to select imagedirectory }
begin
  SelectDirectory(fImageDir,[],0);
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectMemoName;
{ ************************************************************************** }
{ Allow user to select MemoName }
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Text Files|*.TXT|All Files|*.*';
  if MyOpenDialog.execute then MemoName:=MyOpenDialog.Filename;
  MyOpenDialog.Free;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectSortFields;
{ ************************************************************************** }
{ Allow user to select SortFields }
begin
  SortForm(true);
end;


{ ************************************************************************** }
procedure TRRSQL65.SelectGroupFields;
{ ************************************************************************** }
{ Allow user to select GroupFields }
begin
  SortForm(false);
end;

{ ************************************************************************** }
procedure TRRSQL65.SortForm(SortPage:Boolean);
{ ************************************************************************** }
{ Present sortfields/groupfields form depending on SortPage }
var
  GroupForm:TSortDialog;
begin
  GroupForm:=TSortDialog.Create(Application);
  try
    GroupForm.SortGrpNoteBook.PageIndex:=(Ord(not SortPage));
    SortFormLoad(GroupForm);
    if GroupForm.ShowModal=mrOK then
      SortFormSave(GroupForm);
  finally
    GroupForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SortFormLoad(GroupForm:TSortDialog);
{ ************************************************************************** }
{ Load the Sortfields/Groupfields properties into the custom property dialog }
var
  i,myptr:Integer;
  look4:String;
  MyCombo:TComboBox;
  MyCheck:TCheckBox;
begin
   with GroupForm do begin
    { Load field list into dialogs combo's }
    for i:=0 to (fields.count-1) do begin
      SortCombo1.items.add(fields[i]);
      SortCombo2.items.add(fields[i]);
      SortCombo3.items.add(fields[i]);
      SortCombo4.items.add(fields[i]);
      SortCombo5.items.add(fields[i]);
      SortCombo6.items.add(fields[i]);
      SortCombo7.items.add(fields[i]);
      SortCombo8.items.add(fields[i]);
      GroupCombo1.items.add(fields[i]);
      GroupCombo2.items.add(fields[i]);
      GroupCombo3.items.add(fields[i]);
      GroupCombo4.items.add(fields[i]);
      GroupCombo5.items.add(fields[i]);
      GroupCombo6.items.add(fields[i]);
      GroupCombo7.items.add(fields[i]);
      GroupCombo8.items.add(fields[i]);
    end;

    { start at itemindex 0}
    SortCombo1.itemindex:=0;
    SortCombo2.itemindex:=0;
    SortCombo3.itemindex:=0;
    SortCombo4.itemindex:=0;
    SortCombo5.itemindex:=0;
    SortCombo6.itemindex:=0;
    SortCombo7.itemindex:=0;
    SortCombo8.itemindex:=0;
    GroupCombo1.itemindex:=0;
    GroupCombo2.itemindex:=0;
    GroupCombo3.itemindex:=0;
    GroupCombo4.itemindex:=0;
    GroupCombo5.itemindex:=0;
    GroupCombo6.itemindex:=0;
    GroupCombo7.itemindex:=0;
    GroupCombo8.itemindex:=0;

    { Set correct Sortfield itemindex for each combo }
    for i:=0 to (SortFields.count-1) do begin
      if i<8 then begin
        case i of
          0 : begin MyCombo:=SortCombo1;MyCheck:=SortCheck1;end;
          1 : begin MyCombo:=SortCombo2;MyCheck:=SortCheck2;end;
          2 : begin MyCombo:=SortCombo3;MyCheck:=SortCheck3;end;
          3 : begin MyCombo:=SortCombo4;MyCheck:=SortCheck4;end;
          4 : begin MyCombo:=SortCombo5;MyCheck:=SortCheck5;end;
          5 : begin MyCombo:=SortCombo6;MyCheck:=SortCheck6;end;
          6 : begin MyCombo:=SortCombo7;MyCheck:=SortCheck7;end;
          7 : begin MyCombo:=SortCombo8;MyCheck:=SortCheck8;end;
        end;
        look4:=SortFields[i];
        myptr:=MyCombo.items.indexof(Copy(look4,2,255));
        if myptr>0 then MyCombo.itemindex:=myptr;
        MyCheck.Checked:=(Copy(look4,1,1)='+');
      end;
    end;

    { Set correct Groupfield itemindex for each combo }
    for i:=0 to (GroupFields.count-1) do begin
      if i<8 then begin
        case i of
          0 : MyCombo:=GroupCombo1;
          1 : MyCombo:=GroupCombo2;
          2 : MyCombo:=GroupCombo3;
          3 : MyCombo:=GroupCombo4;
          4 : MyCombo:=GroupCombo5;
          5 : MyCombo:=GroupCombo6;
          6 : MyCombo:=GroupCombo7;
          7 : MyCombo:=GroupCombo8;
        end;
        look4:=GroupFields[i];
        myptr:=MyCombo.items.indexof(look4);
        if myptr>0 then MyCombo.itemindex:=myptr;
      end;
    end;

  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SortFormSave(GroupForm:TSortDialog);
{ ************************************************************************** }
{ Save settings made on sortfields/groupfields dialog back to fields property}
var
  Ascend:Array[0..1] of Char;
begin
  Ascend[0]:='-';
  Ascend[1]:='+';
  SortFields.Clear;
  GroupFields.Clear;

  with GroupForm do begin
    if SortCombo1.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck1.Checked)]+SortCombo1.Text);
    if SortCombo2.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck2.Checked)]+SortCombo2.Text);
    if SortCombo3.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck3.Checked)]+SortCombo3.Text);
    if SortCombo4.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck4.Checked)]+SortCombo4.Text);
    if SortCombo5.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck5.Checked)]+SortCombo5.Text);
    if SortCombo6.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck6.Checked)]+SortCombo6.Text);
    if SortCombo7.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck7.Checked)]+SortCombo7.Text);
    if SortCombo8.ItemIndex>0 then
      SortFields.Add(Ascend[ord(SortCheck8.Checked)]+SortCombo8.Text);

    if GroupCombo1.ItemIndex>0 then GroupFields.Add(GroupCombo1.Text);
    if GroupCombo2.ItemIndex>0 then GroupFields.Add(GroupCombo2.Text);
    if GroupCombo3.ItemIndex>0 then GroupFields.Add(GroupCombo3.Text);
    if GroupCombo4.ItemIndex>0 then GroupFields.Add(GroupCombo4.Text);
    if GroupCombo5.ItemIndex>0 then GroupFields.Add(GroupCombo5.Text);
    if GroupCombo6.ItemIndex>0 then GroupFields.Add(GroupCombo6.Text);
    if GroupCombo7.ItemIndex>0 then GroupFields.Add(GroupCombo7.Text);
    if GroupCombo8.ItemIndex>0 then GroupFields.Add(GroupCombo8.Text);
  end;

end;

{ ************************************************************************** }
procedure TRRSQL65.SelectUserParams;
{ ************************************************************************** }
{ Allow user to edit user paramaters }
var
  UserForm:TUserForm;
  i:Integer;
begin
  UserForm:=TUserForm.Create(Application);
  try

    { Load }
    for i:=0 to (UserParamsNames.Count-1) do begin
      UserForm.NamesList.lines.add(UserParamsNames[i]);
    end;
    for i:=0 to (UserParamsValues.Count-1) do begin
      UserForm.ValuesList.lines.add(UserParamsValues[i]);
    end;

    { Display }
    if UserForm.ShowModal=mrOK then begin
      { Save }
      UserParamsNames.Clear;
      for i:=0 to (UserForm.NamesList.lines.Count-1) do begin
        UserParamsNames.Add(UserForm.NamesList.lines[i]);
      end;
      UserParamsValues.Clear;
      for i:=0 to (UserForm.ValuesList.lines.Count-1) do begin
        UserParamsValues.Add(UserForm.ValuesList.lines[i]);
      end;
    end;

  finally
    UserForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.Clear;
{ ************************************************************************** }
{ Clear all properties }
begin
  fDataSource:='';
  fAskPrinter:=false;
  fBeginPage:=0;
  fEndPage:=0;
  fCopies:=1;
  fDatabaseName:='';
  fDataDir:='';
  fErrorCode:='';
  fErrorMessage:='';
  fFields.Clear;
  fFilter:='';
  fFilterUsage:=rrfuSaved;
  fGroupFields.Clear;
  fImageDir:='';
  fLastErrorPage:=0;
  fLibName:='';
  fMasterTableName:='';
  fMemoName:='';
  fOutPutDest:=rrdDisplay;
  fOutPutFile:='';
  fPrinterName:='';
  fPrinterPort:='';
  fRepName:='';
  fReportPick:=false;
  fSortFields.Clear;
  fUserParamsNames.Clear;
  fUserParamsValues.Clear;
  fWinTitle:='';
end;



{ ************************************************************************** }
procedure TRRSQL65.Preview;
{ ************************************************************************** }
{ Preview the report to screen }
var
  oldprinter,oldport,oldoutputFile:String;
  olddest:TrrDest;
  oldwait,olddisplaystatus,oldstatuseverypage,oldaskprinter:Boolean;
begin
  oldaskprinter:=askprinter;
  oldprinter:=printername;
  oldport:=printerport;
  olddest:=outputdest;
  oldoutputfile:=outputfile;
  oldwait:=wait;
  olddisplaystatus:=displaystatus;
  oldstatuseverypage:=statuseverypage;

  { Clear out printername in case not known on this system }
  askprinter:=false;
  printername:='';
  printerport:='';
  outputfile:='';
  wait:=true;
  displaystatus:=true;
  statuseverypage:=true;

  outputdest:=rrdDisplay;
  execute;

  askprinter:=oldaskprinter;
  printername:=oldprinter;
  printerport:=oldport;
  outputdest:=olddest;
  OutputFile:=oldoutputfile;
  wait:=oldwait;
  displaystatus:=olddisplaystatus;
  statuseverypage:=oldstatuseverypage;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectDirs;
{ ************************************************************************** }
{ Allow user to edit default directories }
var
  DirForm:TRRXDirs;
begin
  DirForm:=TRRXDirs.Create(Application);
  try
    { load }
    DirForm.DataDir.text:=fDataDir;
    DirForm.ImageDir.text:=fImageDir;
    { display }
    if DirForm.ShowModal=mrok then begin
      { save }
      fDataDir:=DirForm.DataDir.text;
      fImageDir:=DirForm.ImageDir.text;
    end;
  finally
    DirForm.Free;
  end;
end;


{ ************************************************************************** }
procedure TRRSQL65.SelectDataSource;
{ ************************************************************************** }
{ Select datasource }
var
  hReport:Integer;
  DataSource_,AppName_,ReportLibrary_,ReportName_:PChar;
begin
  DataSource_:=StrAlloc(256);
  AppName_:=StrAlloc(256);
  ReportLibrary_:=StrAlloc(256);
  ReportName_:=StrAlloc(256);

  try
    StrPCopy(DataSource_,'');
    StrPCopy(AppName_,AppName);
    StrPcopy(ReportLibrary_,ReportLibrary);
    StrPCopy(ReportName_,ReportName);

    hReport:=ChooseReport(AppName_,ReportLibrary_,ReportName_,StrBufSize(ReportName_));

    if hReport>0 then begin
      if ChooseDataSource(hReport,DataSource_,StrBufSize(DataSource_)) then
        DataSource:=StrPas(DataSource_);
      endReport(hReport);
    end;

  finally
    StrDispose(DataSource_);
    StrDispose(AppName_);
    StrDispose(ReportLibrary_);
    StrDispose(ReportName_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectDatabase;
{ ************************************************************************** }
{ Select database }
var
  hReport:Integer;
  Table_,Database_,DataSource_,AppName_,ReportLibrary_,ReportName_:PChar;
begin
  Database_:=StrAlloc(256);
  Table_:=StrAlloc(256);
  DataSource_:=StrAlloc(256);
  AppName_:=StrAlloc(256);
  ReportLibrary_:=StrAlloc(256);
  ReportName_:=StrAlloc(256);

  try
    StrPCopy(Database_,Databasename);
    StrPCopy(DataSource_,DataSource);
    StrPCopy(AppName_,AppName);
    StrPcopy(ReportLibrary_,ReportLibrary);
    StrPCopy(ReportName_,ReportName);

    hReport:=ChooseReport(AppName_,ReportLibrary_,ReportName_,StrBufSize(ReportName_));

    if hReport>0 then begin
      if ChooseTable(hReport,Table_,StrBufSize(Table_),DataSource_,
                     StrBufSize(DataSource_),Database_,StrBufSize(Database_)) then
        DatabaseName:=StrPas(Database_);
      endReport(hReport);
    end;

  finally
    StrDispose(Database_);
    StrDispose(Table_);
    StrDispose(DataSource_);
    StrDispose(AppName_);
    StrDispose(ReportLibrary_);
    StrDispose(ReportName_);
  end;
end;


{ ************************************************************************** }
procedure TRRSQL65.SelectFilter;
{ ************************************************************************** }
{ Select filter }
{ Allow user to edit filter and scope }
var
  FilterForm:TRRSQLFilt;
  i:integer;
begin
  FilterForm:=TRRSQLFilt.Create(Application);
  try

    { Load info into dialog }
    FilterForm.FilterMemo.lines.assign(FilterLong);
    FilterForm.WhereMemo.Text:=Where;

    FilterForm.ReplaceMemo.clear;
    for i:=0 to (Replaces.Count-1) do
      FilterForm.ReplaceMemo.lines.add(Replaces[i]);


    if FilterUsage=rrfuSaved       then FilterForm.FilterUsageCombo.Itemindex:=0;
    if FilterUsage=rrfuComponent   then FilterForm.FilterUsageCombo.Itemindex:=1;
    if FilterUsage=rrfuInteractive then FilterForm.FilterUsageCombo.Itemindex:=2;

    { Display }
    if FilterForm.ShowModal=mrok then begin
      { Save }
      ffilterlong.assign(FilterForm.FilterMemo.lines);
      if ffilterlong.count>0 then
        ffilter:=ffilterlong[0]
      else
        ffilter:='';

      where:=FilterForm.WhereMemo.Text;

      Replaces.clear;
      for i:=0 to (FilterForm.ReplaceMemo.Lines.Count-1) do
        Replaces.add(FilterForm.ReplaceMemo.Lines[i]);

      if FilterForm.FilterUsageCombo.ItemIndex=0 then fFilterUsage:=rrfuSaved;
      if FilterForm.FilterUsageCombo.ItemIndex=1 then fFilterUsage:=rrfuComponent;
      if FilterForm.FilterUsageCombo.ItemIndex=2 then fFilterUsage:=rrfuInteractive;

    end;

  finally
    FilterForm.Free;
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SelectJoins;
{ ************************************************************************** }
{ Allow user to edit joins }
var
  RelationForm:TJoinForm;
  i,ListCount:Integer;
begin
  RelationForm:=TJoinForm.Create(Application);
  try
    with RelationForm do begin
      { load }
      Report:=Self;
      MasterTableEdit.Text:=MasterTableName;

      ListCount:=JoinAliasNames.Count;
      if JoinTableNames.Count>ListCount then
        ListCount:=JoinTableNames.Count;

      for i:=0 to (JoinAliasNames.Count-1) do begin
        AliasCombo.Items.Add(JoinAliasNames[i]);
      end;
      for i:=1 to (ListCount-JoinAliasNames.Count) do begin
        AliasCombo.Items.Add('');
      end;

      for i:=0 to (JoinTableNames.Count-1) do begin
        TableList.Items.Add(JoinTableNames[i]);
      end;
      for i:=1 to (ListCount-JoinTableNames.Count) do begin
        TableList.Items.Add('');
      end;

      if ListCount>0 then begin
        AliasCombo.ItemIndex:=0;
        TableEdit.text:=TableList.Items[0];
      end;

      if ListCount=0 then begin
        AliasLabel.enabled:=false;
        AliasCombo.enabled:=false;
        TableLabel.enabled:=false;
        TableEdit.enabled:=false;
        TableBtn.enabled:=false;
      end;

      { display }
      if ShowModal=mrOK then begin

        { save }
        MasterTableName:=MasterTableEdit.Text;

        JoinAliasNames.Clear;
        JoinTableNames.Clear;
        for i:=0 to (AliasCombo.Items.Count-1) do begin
          JoinAliasNames.Add(AliasCombo.Items[i]);
          JoinTableNames.Add(TableList.Items[i]);
        end;

      end;

    end;
  finally
    RelationForm.Free;
  end;
end;

{ ************************************************************************** }
function TRRSQL65.SelectTable:String;
{ ************************************************************************** }
{ return a tablename.}
var
  hReport:Integer;
  Table_,Database_,DataSource_,AppName_,ReportLibrary_,ReportName_:PChar;
begin
  Database_:=StrAlloc(256);
  Table_:=StrAlloc(256);
  DataSource_:=StrAlloc(256);
  AppName_:=StrAlloc(256);
  ReportLibrary_:=StrAlloc(256);
  ReportName_:=StrAlloc(256);

  try
    Result:='';
    StrPCopy(Database_,Databasename);
    StrPCopy(DataSource_,DataSource);
    StrPCopy(AppName_,AppName);
    StrPcopy(ReportLibrary_,ReportLibrary);
    StrPCopy(ReportName_,ReportName);

    hReport:=ChooseReport(AppName_,ReportLibrary_,ReportName_,StrBufSize(ReportName_));

    if hReport>0 then begin
      if ChooseTable(hReport,Table_,StrBufSize(Table_),DataSource_,
                     StrBufSize(DataSource_),Database_,StrBufSize(Database_)) then
        Result:=StrPas(Table_);
      endReport(hReport);
    end;

  finally
    StrDispose(Database_);
    StrDispose(Table_);
    StrDispose(DataSource_);
    StrDispose(AppName_);
    StrDispose(ReportLibrary_);
    StrDispose(ReportName_);
  end;
end;

{ ************************************************************************** }
procedure TRRSQL65.SetVersion(Value: String);
{ ************************************************************************** }
{ Dont allow anybody to change the version number }
begin
  fVersion:='2.3';
end;


{ ****************** Code For Property Editors Follows ******************** }


{ Reportlibray property editor code }
procedure TRLPRopertyEditor.edit;
var
  MyOpenDialog:TOpenDialog;
begin
  MyOpenDialog:=TOpenDialog.Create(Application);
  MyOpenDialog.Filter:='Report Librarys|*.RP6';
  if MyOpenDialog.execute then SetStrValue(MyOpenDialog.Filename);
  MyOpenDialog.Free;
end;
function TRLPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TRLPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TRLPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Reportname property editor code }
procedure TRNPropertyEditor.edit;
begin
 TRRSQL65(GetComponent(0)).SelectReportName;
end;
function TRNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TRNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TRNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Printername/Printerport Property Editor Code }
procedure TPRPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectPrinter;
end;
function TPRPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TPRPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TPRPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Datadirectory Property Editor Code }
procedure TDDPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectDataDirectory;
end;
function TDDPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TDDPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TDDPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;


{ ImageDirectory property Editor Code }
procedure TIDPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectImageDirectory;
end;
function TIDPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TIDPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TIDPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Memoname Property Editor Code }
procedure TMNPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectMemoName;
end;
function TMNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TMNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TMNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Sortfields property editor code }
procedure TSFPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectSortFields;
end;
function TSFPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TSFPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TSFPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ Groupfields property editor code }
procedure TGFPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectGroupFields;
end;
function TGFPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TGFPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TGFPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ UserParamsNames/Values property editor code }
procedure TUSPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectUserParams;
end;
function TUSPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TUSPropertyEditor.GetValue:String;
begin
  Result:='(TStrings)';
end;
procedure TUSPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue('(TStrings)');
end;

{ Datasource property Editor Code }
procedure TDSPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectDataSource;
end;
function TDSPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TDSPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TDSPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Datasource property Editor Code }
procedure TDNPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectDatabase;
end;
function TDNPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TDNPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TDNPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

{ Filter property editor code }
procedure TFIPropertyEditor.edit;
begin
  TRRSQL65(GetComponent(0)).SelectFilter;
end;
function TFIPropertyEditor.getattributes:TPropertyAttributes;
begin
  result:=[paDialog];
end;
function TFIPropertyEditor.GetValue:String;
begin
  Result:=GetStrValue;
end;
procedure TFIPropertyEditor.SetValue(Const Value:String);
begin
  SetStrValue(Value);
end;

end.
